<?php
/**
 * DB controller
 *
 * @package wpjb
 * @subpackage controllers
 * @since WP Jobs Board 1.0
 */

namespace Wpjb\Controllers;

defined( 'ABSPATH' ) or die( 'No script kiddies please!' );
use Wpjb\Controllers\WpjbHelper as Helper;

/**
 * Class DB Controller
 *
 * @category Controller
 * @package wpjb
 * @author  IV | PIXOLETTE
 * @license  http://wp.pixolette.com
 * @link     http://wp.pixolette.com
 */
class WpjbDb {

	public $wpjb_db_version;
	protected $table_name;
	protected $charset_collate;

	/**
	 * Function to be called on class init
	 */
	function __construct() {
		global $wpdb;

		$this->wpjb_db_version = '1.2';
		$this->charset_collate  = $wpdb->get_charset_collate();
		// table to store data
		$this->table_jobs = $wpdb->prefix . 'wpjb_jobs_data';
		$this->table_applications = $wpdb->prefix . 'wpjb_applications';
		$this->table_favourites = $wpdb->prefix . 'wpjb_favourites';
		$this->table_views = $wpdb->prefix . 'wpjb_views';
	}

	/**
	 * Function to create tables on plugin install
	 */
	function create_table_on_install() {
		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

		$sql_jobs = "CREATE TABLE $this->table_jobs (
			id int NOT NULL AUTO_INCREMENT,
			timestamp int NOT NULL,
			job_id int NOT NULL,
			lat float (10,6) NOT NULL,
			lng float (10,6) NOT NULL,
			status int DEFAULT 0 NULL,
			PRIMARY KEY (id)
		) $this->charset_collate;";

		dbDelta( $sql_jobs );

		$sql_applications = "CREATE TABLE $this->table_applications (
			id int NOT NULL AUTO_INCREMENT,
			job_id int NOT NULL,
			timestamp int NOT NULL,
			user_id int NOT NULL,
			attachment_id int NULL,
			attachment_url text NULL,
			user_name varchar(255) NULL,
			user_email varchar(255) NULL,
			user_message text NULL,
			user_custom_data text NULL,
			status int DEFAULT 1 NULL,
			PRIMARY KEY (id)
		) $this->charset_collate;";

		dbDelta( $sql_applications );

		$sql_favourites = "CREATE TABLE $this->table_favourites (
			id int NOT NULL AUTO_INCREMENT,
			job_id int NOT NULL,
			timestamp int NOT NULL,
			user_id int NOT NULL,
			status int DEFAULT 1 NULL,
			PRIMARY KEY (id)
		) $this->charset_collate;";

		dbDelta( $sql_favourites );

		$sql_views = "CREATE TABLE $this->table_views (
			id int NOT NULL AUTO_INCREMENT,
			job_id int NOT NULL,
			timestamp int NOT NULL,
			user_id int NOT NULL,
			status int DEFAULT 1 NULL,
			PRIMARY KEY (id)
		) $this->charset_collate;";

		dbDelta( $sql_views );

		if ( get_option( 'wpjb_db_version' ) ) {
			update_option( 'wpjb_db_version', $this->wpjb_db_version );
		} else {
			add_option( 'wpjb_db_version', $this->wpjb_db_version );
		}
	}

	/**
	 * Function to create entry
	 *
	 * @param mixed $data .
	 * @param mixed $format .
	 */
	function create_job_entry( $data ) {
		global $wpdb;
		$res = $wpdb->insert( $this->table_jobs, $data );

		return $wpdb->insert_id;
	}

	/**
	 * Function to update entry
	 *
	 * @param mixed $data .
	 * @param int $id .
	 * @param mixed $format .
	 */
	function update_job_entry( $data, $id ) {
		global $wpdb;
		$updated = $wpdb->update( $this->table_jobs, $data, array( 'id' => (int) $id ) );
		return $updated;
	}

	/**
	 * Function to update entry
	 *
	 * @param mixed $data .
	 * @param int $id .
	 * @param mixed $format .
	 */
	function create_or_update_coords_by_job_id( $data, $job_id ) {
		global $wpdb;
		$sql = $wpdb->prepare( "SELECT * FROM $this->table_jobs WHERE job_id = %d", intval( $job_id ) );
		$entry = $wpdb->get_row( $sql, ARRAY_A );

		if ( $entry ) {
			$entry['lat'] = $data['lat'];
			$entry['lng'] = $data['lng'];
			$entry['status'] = 1;
			$status = $this->update_job_entry( $data, $entry['id'] );
			if ( $status ) {
				return $entry['id'];
			}
		} else {
			$data['timestamp'] = time();
			$data['job_id'] = $job_id;
			$data['status'] = 1;
			$entry_id = $this->create_job_entry( $data );
			return $entry_id;
		}
	}

	function get_entries_by_coords( $coords, $radius ) {
		global $wpdb;
		$settings = Helper::get_settings();
		// 6371 = km ; 3959 = miles
		if ( 'mi' == $settings['wpjb-gmaps-unit'] ) {
			$rval = 3959;
		} else {
			$rval = 6371;
		}
		$sql     = $wpdb->prepare( "SELECT id, job_id, status, ( %d * acos( cos( radians(%s) ) * cos( radians( lat ) ) * cos( radians( lng ) - radians(%s) ) + sin( radians(%s) ) * sin( radians( lat ) ) ) ) AS distance FROM $this->table_jobs HAVING distance < %d AND status = %d ORDER BY distance", $rval, $coords['lat'], $coords['lng'], $coords['lat'], $radius, 1 );
		$entries = $wpdb->get_results( $sql, ARRAY_A );

		if ( ! empty( $entries ) ) {
			$to_return = array_map(
				function ( $value ) {
					return $value['job_id'];
				},
				$entries
			);

			return $to_return;
		}
	}

	/**
	 * Function to create entry
	 *
	 * @param mixed $data .
	 * @param mixed $format .
	 */
	function create_job_application_entry( $data ) {
		global $wpdb;
		$res = $wpdb->insert( $this->table_applications, $data );

		return $wpdb->insert_id;
	}

	/**
	 * Function to update entry
	 *
	 * @param mixed $data .
	 * @param int $id .
	 * @param mixed $format .
	 */
	function update_job_application_entry( $data, $id ) {
		global $wpdb;
		$updated = $wpdb->update( $this->table_applications, $data, array( 'id' => (int) $id ) );
		return $updated;
	}

	/**
	 * Function to update entry
	 *
	 * @param mixed $data .
	 * @param int $id .
	 * @param mixed $format .
	 */
	function get_all_applications( $page_nr = 1, $per_page = 20 ) {
		global $wpdb;
		$offset  = ( $page_nr - 1 ) * $per_page;
		$sql = $wpdb->prepare( "SELECT * FROM $this->table_applications ORDER BY id DESC LIMIT %d, %d", $offset, $per_page );
		$entries = $wpdb->get_results( $sql, ARRAY_A );
		return $entries;
	}

	/**
	 * Function to update entry
	 *
	 * @param mixed $data .
	 * @param int $id .
	 * @param mixed $format .
	 */
	function get_all_applications_pages_nr( $per_page = 20 ) {
		global $wpdb;
		$total_pages = 1;
		$sql = $wpdb->prepare( "SELECT count(id) as total_entries FROM $this->table_applications WHERE id > %d", 0 );
		$entries = $wpdb->get_row( $sql, ARRAY_A );

		if ( 0 < $entries['total_entries'] ) {
			$total_pages = ceil( $entries['total_entries'] / $per_page );
		}
		return $total_pages;
	}

	/**
	 * Function to update entry
	 *
	 * @param mixed $data .
	 * @param int $id .
	 * @param mixed $format .
	 */
	function get_applications_by_job_id( $job_id, $page_nr = 1, $per_page = 20 ) {
		global $wpdb;
		$offset  = ( $page_nr - 1 ) * $per_page;
		$sql = $wpdb->prepare( "SELECT * FROM $this->table_applications WHERE job_id = %d ORDER BY id DESC LIMIT %d, %d", intval( $job_id ), $offset, $per_page );
		$entries = $wpdb->get_results( $sql, ARRAY_A );
		return $entries;
	}

	/**
	 * Function to update entry
	 *
	 * @param mixed $data .
	 * @param int $id .
	 * @param mixed $format .
	 */
	function get_applications_pages_nr_by_job_id( $job_id, $per_page = 20 ) {
		global $wpdb;
		$total_pages = 1;
		$sql = $wpdb->prepare( "SELECT count(id) as total_entries FROM $this->table_applications WHERE job_id = %d", intval( $job_id ) );
		$entries = $wpdb->get_row( $sql, ARRAY_A );
		if ( 0 < $entries['total_entries'] ) {
			$total_pages = ceil( $entries['total_entries'] / $per_page );
		}
		return $total_pages;
	}

	/**
	 * Function to update entry
	 *
	 * @param mixed $data .
	 * @param int $id .
	 * @param mixed $format .
	 */
	function get_applications_number_for_jobs( $job_ids ) {
		global $wpdb;

		$in_val = array_map(function($v) {
			return "'" . esc_sql($v) . "'";
		}, $job_ids);
		$in = implode(',', $in_val);
		$sql = $wpdb->prepare( "SELECT job_id, count( id ) as job_applications FROM $this->table_applications WHERE job_id IN ( $in ) AND timestamp > %d GROUP BY job_id", 1 );
		$entries = $wpdb->get_results( $sql, ARRAY_A );
		$results = array();
		if ( ! empty( $entries ) ) {
			foreach ( $entries as $key => $value ) {
				$results[ $value['job_id'] ] = $value['job_applications'];
			}
		}

		return $results;
	}

	/**
	 * Function to update entry
	 *
	 * @param mixed $data .
	 * @param int $id .
	 * @param mixed $format .
	 */
	function get_applications_for_candidate( $page_nr = 1, $per_page = 20 ) {
		if ( is_user_logged_in() && current_user_can( 'candidate' ) ) {
			global $wpdb;
			$user_id = get_current_user_id();
			$offset  = ( $page_nr - 1 ) * $per_page;
			$sql = $wpdb->prepare( "SELECT * FROM $this->table_applications WHERE user_id = %d ORDER BY id DESC LIMIT %d, %d", intval( $user_id ), $offset, $per_page );
			$entries = $wpdb->get_results( $sql, ARRAY_A );
			return $entries;
		}
	}

	/**
	 * Function to update entry
	 *
	 * @param mixed $data .
	 * @param int $id .
	 * @param mixed $format .
	 */
	function get_applications_pages_nr_for_candidate( $per_page = 20 ) {
		if ( is_user_logged_in() && current_user_can( 'candidate' ) ) {
			global $wpdb;
			$user_id = get_current_user_id();
			$total_pages = 1;
			$sql = $wpdb->prepare( "SELECT count(id) as total_entries FROM $this->table_applications WHERE user_id = %d", intval( $user_id ) );
			$entries = $wpdb->get_row( $sql, ARRAY_A );
			if ( 0 < $entries['total_entries'] ) {
				$total_pages = ceil( $entries['total_entries'] / $per_page );
			}
			return $total_pages;
		}
	}

	function create_or_delete_favourite_job( $job_id, $user_id ) {
		global $wpdb;
		$sql = $wpdb->prepare( "SELECT * FROM $this->table_favourites WHERE job_id = %d AND user_id = %d", (int) $job_id, (int) $user_id );
		$entry = $wpdb->get_row( $sql, ARRAY_A );

		if ( $entry && isset( $entry['id'] ) ) {
			$deleted = $this->delete_favourite_entry( $entry['id'] );
			return $deleted;
		} else {
			$data = array(
				'job_id'    => $job_id,
				'user_id'   => $user_id,
				'timestamp' => time(),
			);
			$id = $this->create_favourite_entry( $data );
			return $id;
		}
	}

	/**
	 * Function to create entry
	 *
	 * @param mixed $data .
	 * @param mixed $format .
	 */
	function create_favourite_entry( $data ) {
		global $wpdb;
		$res = $wpdb->insert( $this->table_favourites, $data );

		return $wpdb->insert_id;
	}


	/**
	 * Function to delete entry
	 *
	 * @param mixed $data .
	 * @param int $id .
	 * @param mixed $format .
	 */
	function delete_favourite_entry( $id ) {
		global $wpdb;
		$deleted = $wpdb->delete( $this->table_favourites, array( 'id' => (int) $id ) );
		return $deleted;
	}

	function get_favourite_jobs_for_current_user() {
		global $wpdb;
		$user_id = get_current_user_id();
		$sql = $wpdb->prepare( "SELECT * FROM $this->table_favourites WHERE user_id = %d", (int) $user_id );
		$entries = $wpdb->get_results( $sql, ARRAY_A );

		return $entries;
	}

}
